@@ -10,6 +10,7 @@ import os
 import dateutil
 from datetime import datetime
 import re
+import glob
 
 class bcolors:
     HEADER = '\033[95m'
@@ -22,18 +23,56 @@ class bcolors:
     UNDERLINE = '\033[4m'
 
 class AgentLifeCycle:
-    _docker_container_names = ["cgs_agent","remote_intervention_agent","ecs_server","rosbridge"]
+    _docker_container_names = ["cgs_agent","cgs_diagnostics_agent","remote_intervention_agent","ecs_server","cgs_diagnostics_ecs_api","cgs_diagnostics_streamer_api","rosbridge"]
     _docker_images = {}
-    _docker_images["remote_intervention_agent"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/remote_intervention_agent"
-    _docker_images["cgs_agent"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_rosbridge_listener_agent"
-    _docker_images["ecs_server"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_rosbridge_listener_agent"
-    _docker_images["rosbridge"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/orbitty"
-    _docker_images["orbitty"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/orbitty"
+    _docker_images["remote_intervention_agent"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/remote_intervention_agent:latest"
+    _docker_images["cgs_agent"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_rosbridge_listener_agent:latest"
+    _docker_images["cgs_diagnostics_agent"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_diagnostics_agent:latest"
+    _docker_images["ecs_server"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_rosbridge_listener_agent:latest"
+    _docker_images["cgs_diagnostics_ecs_api"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_diagnostics_api:latest"
+    _docker_images["cgs_diagnostics_streamer_api"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_diagnostics_api:latest"
+    _docker_images["rosbridge"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/orbitty:latest"
+    _docker_images["orbitty"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/orbitty:latest"
+    
+    def configure_containers(object, cfg):
+        if("COG_AGENT_CONTAINERS" in cfg.config and "COG_AGENT_IMAGES" in cfg.config):
+            container_names = cfg.config["COG_AGENT_CONTAINERS"].split(";")
+            image_names = cfg.config["COG_AGENT_IMAGES"].split(";")
+            if(len(image_names) == len(container_names)):
+                object._docker_container_names = container_names
+                object._docker_images = {}
+                i = 0
+                for container_name in container_names:
+                    object._docker_images[container_name] = image_names[i]
+                    i = i + 1
+            else:
+                print("`COG_AGENT_CONTAINERS` and `COG_AGENT_IMAGES` do not coincide. Using default.")
+        else:
+            print("Undefined `COG_AGENT_CONTAINERS` or `COG_AGENT_IMAGES`. Using default.")
+
+    def get_latest_log_loc(object, args):
+        # get latest log location
+        latest_log_loc_file_path = os.path.expanduser(args.path+"agent/logs/latest_log_loc.txt")
+        latest_log_loc = ""
+        try:            
+            with open(latest_log_loc_file_path) as txt_file:                
+                latest_log_loc_temp = txt_file.readline()
+                latest_log_loc_temp = latest_log_loc_temp[:-1]
+                latest_log_loc = latest_log_loc_temp.replace("/$HOME/.cognicept/","")
+        except:
+            cgs_agent_status = f"{bcolors.FAIL}UNKNOWN{bcolors.ENDC}"
+        
+        return latest_log_loc
+
     def get_status(object, args):
         client = docker.from_env()
 
         # check status of cgs_agent
-        file_path = os.path.expanduser(args.path+"logs/logDataStatus.json")        
+        # get latest log location
+        latest_log_loc = object.get_latest_log_loc(args)
+        
+        # read latest status and set for display
+        file_path = os.path.expanduser(args.path+latest_log_loc+"/logDataStatus.json")        
         try:            
             with open(file_path) as json_file:                
                 data = json.load(json_file)
@@ -57,19 +96,45 @@ class AgentLifeCycle:
                     if(container_name == "cgs_agent"):
                         print(cgs_agent_status)
                     elif(container_name == "remote_intervention_agent"):
-                        object.parseRemoteInterventionAgentLogs(container.logs(tail=50))                        
+                        object.parseRemoteInterventionAgentLogs(container.logs(tail=50))
+                    elif(container_name == "cgs_diagnostics_agent"):
+                        print(cgs_agent_status)                        
                     else:
                         print(f"{bcolors.OKBLUE}ONLINE{bcolors.ENDC}")
             except docker.errors.NotFound:
                 print(f"{bcolors.FAIL}CONTAINER NOT FOUND{bcolors.ENDC}")
     
+    def get_last_event(object, args):
+        
+        # get latest log location
+        latest_log_loc = object.get_latest_log_loc(args)
+        
+        # read and display latest log if any
+        file_path = os.path.expanduser(args.path+latest_log_loc)        
+        try:
+            print(f"{bcolors.OKBLUE}Looking for the latest event log...{bcolors.ENDC}")
+            # should read only latest logData#.json file and not logDataStatus.json
+            list_of_log_files = [fn for fn in glob.glob(file_path + "/*.json") \
+                                    if not os.path.basename(fn).endswith("logDataStatus.json")]
+            latest_log_file = max(list_of_log_files, key=os.path.getctime)            
+            
+            with open(latest_log_file) as json_file:                
+                data = json.load(json_file)
+                print(f"{bcolors.OKGREEN}Latest Event Log:{bcolors.ENDC}")
+                print(json.dumps(data, indent=4, sort_keys=True))                
+        except:
+            print(f"{bcolors.WARNING}No event logs present.{bcolors.ENDC}")
+
     def parseRemoteInterventionAgentLogs(object, logs):
+        ## Parses logs to find status of remote intervention agent
         logs_lines = logs.splitlines()
         # parse logs to get current status
         ri_agent_status = {}
         ri_agent_status["AGENT"] = ""
         ri_agent_status["WEBRTC"] = ""
         ri_agent_status["WEBSOCKET"] = ""
+
+        # find latest status of the each module (agent, webrtc, websocket)
         for line in reversed(logs_lines):
             for key,value in ri_agent_status.items():
                 if(value != ""):
@@ -79,14 +144,17 @@ class AgentLifeCycle:
                     ri_agent_status[key] = matches.groups(0)[0]
             if(ri_agent_status["AGENT"] != "" and ri_agent_status["WEBRTC"] != "" and ri_agent_status["WEBSOCKET"] != ""):
                 continue
-        if(ri_agent_status["AGENT"] == "OK" and ri_agent_status["WEBRTC"] == "OK" and ri_agent_status["WEBSOCKET"] == "OK"):
-            print(f"{bcolors.OKBLUE}ONLINE{bcolors.ENDC}")
-        else:
-            for key,value in ri_agent_status.items():
-                if(value == ""):
-                    print(f"{bcolors.WARNING} NOT INITIALIZED {bcolors.ENDC}")
-                if(value != "OK"):
-                    print(f"{bcolors.WARNING} {key} {value} {bcolors.ENDC}")
+        
+        output_text = f"{bcolors.OKBLUE}ONLINE{bcolors.ENDC}"
+        for key,value in ri_agent_status.items():
+            if(value == ""):
+                #if not found, it was not yet initialized
+                output_text = f"{bcolors.WARNING} NOT INITIALIZED {bcolors.ENDC}"
+                break
+            if(value != "OK"):
+                output_text = f"{bcolors.WARNING} {key} {value} {bcolors.ENDC}"
+                break
+        print(output_text)
 
     def restart(object, args):
         print("Restarting agents")
@@ -107,14 +175,15 @@ class AgentLifeCycle:
                 print(f"{bcolors.WARNING}NOT FOUND{bcolors.ENDC}")                	
             except docker.errors.APIError:
                 print(f"{bcolors.FAIL}ERROR{bcolors.ENDC}")
-        
-
+    
     def run(object, args):
-        args.config.load_config(args.path)
         object._agent_run_options = {}
-        object._agent_run_options["cgs_agent"] = {"command": "start_cognicept_agent.py","volumes":{args.config.config_path + "logs/": {"bind": "/app/logs", "mode":"rw"}},"network_mode": "host"}
+        object._agent_run_options["cgs_agent"] = {"command": "start_cognicept_agent.py","volumes":{args.config.config_path + "agent/logs/": {"bind": "/app/logs", "mode":"rw"}},"network_mode": "host"}
+        object._agent_run_options["cgs_diagnostics_agent"] = {"command": "roslaunch rosrect-listener-agent listener-agent.launch" ,"volumes":{args.config.config_path + "agent/logs/": {"bind": "/root/.cognicept/agent/logs", "mode":"rw"}} ,"network_mode": "host"}        
         object._agent_run_options["ecs_server"] = {"command": "/ecs_api_server/ecs_endpoint.py","network_mode": "host"}
-        object._agent_run_options["remote_intervention_agent"] = {"command": "rosrun remote_intervention_agent cognicept_agent_node" ,"network_mode": "host"}
+        object._agent_run_options["cgs_diagnostics_ecs_api"] = {"command": "/ecs_api_server/ecs_endpoint.py" ,"network_mode": "host"}
+        object._agent_run_options["cgs_diagnostics_streamer_api"] = {"command": "/data_streamer/streamer_endpoint.py" ,"network_mode": "host"}
+        object._agent_run_options["remote_intervention_agent"] = {"command": "rosrun remote_intervention_agent cognicept_agent_node" ,"network_mode": "host"}        
         object._agent_run_options["rosbridge"] = {"command": "roslaunch rosbridge_server rosbridge_websocket.launch" ,"network_mode": "host"}  
         client = docker.from_env()
         print("RUN: ")
@@ -136,8 +205,8 @@ class AgentLifeCycle:
                 print(f"{bcolors.WARNING}IMAGE NOT FOUND{bcolors.ENDC} (run `cognicept update`)")
             except docker.errors.APIError:
                 print(f"{bcolors.FAIL}DOCKER ERROR{bcolors.ENDC}")
+    
     def run_orbitty(object, args):
-        args.config.load_config(args.path)
         os.system("xhost +local:root")
         client = docker.from_env()
         try:
@@ -166,7 +235,6 @@ class AgentLifeCycle:
 
     def update(object, args):
         print("Info: This may take a while depending on your connection.")
-        args.config.load_config(args.path)
         ecr_client = boto3.client('ecr', region_name='ap-southeast-1', aws_access_key_id=args.config.config["AWS_ACCESS_KEY_ID"], aws_secret_access_key = args.config.config["AWS_SECRET_ACCESS_KEY"])
         
         
@@ -183,7 +251,7 @@ class AgentLifeCycle:
 
         for container_name, image_name in object._docker_images.items():
             print(image_name)
-            result = docker_client.images.pull(image_name, tag="latest")
+            result = docker_client.images.pull(image_name)
 
         print("Info: Run `cognicept restart` to redeploy updated agents.")
         
