@@ -14,6 +14,7 @@ class Configuration:
     config_path = os.path.expanduser("~/.cognicept/")
     env_path = config_path + "runtime.env"
     _regex_key = r"^([_A-Z0-9]+)$"
+    _config_loaded = False
 
     def load_config(object, path):
         object.config_path = os.path.expanduser(path)
@@ -30,10 +31,11 @@ class Configuration:
             print("Configuration file `" + object.env_path + "` is empty or could not be parsed.")
             return False
 
+        object._config_loaded = True
         return True
 
     def configure(object, args):
-        if(not object.load_config(args.path)):
+        if(not object._config_loaded):
             return
         
         if (not os.access(object.env_path, os.W_OK)):
